@@ -34,7 +34,7 @@ from requests import HTTPError
 from tabulate import tabulate
 
 from dynex.config import DynexConfig
-from interfaces.api import Job, JobOpts
+from dynex.interfaces.api import Job, JobOpts
 
 
 class DynexAPI:
@@ -426,7 +426,7 @@ class DynexAPI:
                                                 """)
                     retval = True
                 if data['billing_type'] == 1:  # pay-per-use pricing
-                    
+
                     retval = True
                     if self.logging:
                         max_chips = data['max_chips']
