@@ -36,8 +36,11 @@ import zlib
 import base64
 import inspect
 import warnings
+import os
+import logging
 
 from dynex import DynexConfig
+from dynex.pennylane_bridge import PennylaneBridge as DynexCircuitBridge
 
 warnings.filterwarnings("ignore", category=DeprecationWarning)
 
@@ -119,6 +122,141 @@ class DynexCircuit:
         with open(filename, "w", encoding="utf-8") as f:
             f.write(dnx_circuit.circuit_str)
 
+    @staticmethod
+    def _convert_qasm_to_bqm(qasm_filepath, qasm_filename, debugging=False):
+        """
+        `Internal Function`
+
+        Converts QASM circuit to BQM for testnet (replaces dynex_circuit_backend.py).
+        This method performs the same conversion as the external script.
+
+        :Parameters:
+        - :qasm_filepath: Path to QASM file directory
+        - :qasm_filename: QASM filename
+        - :debugging: Enable debugging output
+
+        :Returns:
+        - Tuple of (feed_dict, model_dict) containing BQM data
+        """
+        logger = logging.getLogger("dynex.circuit")
+        if not logger.hasHandlers():
+            handler = logging.StreamHandler()
+            formatter = logging.Formatter("[DYNEX] %(levelname)s: %(message)s")
+            handler.setFormatter(formatter)
+            logger.addHandler(handler)
+        logger.setLevel(logging.INFO)
+
+        def _qasm_to_circuit(t, debugging=False):
+            """Convert base85 encoded QASM string to DynexCircuit"""
+            dec = base64.b85decode(t.encode('utf-8'))
+            decomp = zlib.decompress(dec)
+            cir_i = json.loads(decomp.decode('utf-8'))
+
+            def _apply_ops(op, params):
+                b_op = getattr(qml, op["base_name"])
+                op_params = [np.array(p) if isinstance(p, list) else p for p in op["params"]]
+                if op["base_name"] == 'BasisEmbedding':
+                    if "params" in op and op["params"]:
+                        basis_state = np.array([int(x) for x in op["params"][0]])
+                    elif "hyperparams" in op and "basis_state" in op["hyperparams"]:
+                        basis_state = np.array([int(x) for x in op["hyperparams"]["basis_state"]])
+                    operation = lambda wires: b_op(basis_state, wires=wires)
+                elif op["base_name"] == 'FlipSign':
+                    n = op["hyperparams"]["arr_bin"]
+                    operation = lambda wires: b_op(n, wires=wires)
+                elif op["base_name"] == 'ControlledQubitUnitary':
+                    matrix = op_params[0]
+                    c_w = op["control_wires"]
+                    t_w = op["target_wires"]
+                    return b_op(matrix, control_wires=c_w, wires=t_w)
+                elif op["base_name"] == 'QubitUnitary':
+                    matrix = np.array(op["params"][0], dtype=complex)
+                    return b_op(matrix, wires=op["wires"])
+                elif op["base_name"] == 'QuantumPhaseEstimation':
+                    u = op["unitary"]
+                    if isinstance(u, dict):
+                        u_op = getattr(qml, u["name"])
+                        u_params = [np.array(p) if isinstance(p, list) else p for p in u["params"]]
+                        u = u_op(*u_params, wires=u["wires"])
+                        return b_op(u, estimation_wires=op["estimation_wires"])
+                    else:
+                        u = np.array(u)
+                        return b_op(u, target_wires=op["target_wires"], estimation_wires=op["estimation_wires"])
+                elif op_params:
+                    operation = lambda wires: b_op(*op_params, wires=wires)
+                else:
+                    operation = lambda wires: b_op(wires=wires)
+                for _ in range(op["ctrlD"]):
+                    c_w = op["control_wires"]
+                    operation = qml.ctrl(operation, control=c_w)
+                for _ in range(op["adjointD"]):
+                    operation = qml.adjoint(operation)
+                if op["ctrlD"] > 0:
+                    return operation(wires=op["target_wires"])
+                else:
+                    return operation(wires=op["wires"])
+
+            def _rec_circ(params):
+                for op in cir_i["operations"]:
+                    _apply_ops(op, params)
+                return qml.state()
+
+            _rec_circ.n_wires = cir_i["nWires"]
+            _rec_circ.n_params = cir_i["nParams"]
+            dynex_circuit = DynexCircuitBridge(_rec_circ, cir_i["params"], _rec_circ.n_wires, draw=False)
+            return dynex_circuit
+
+        def _compress(instring):
+            return zlib.compress(bytes(instring, 'utf-8')).hex()
+
+        # Read QASM file
+        if os.path.isabs(qasm_filepath):
+            qasm_file_full = qasm_filepath + qasm_filename
+        else:
+            qasm_file_full = os.path.join(qasm_filepath, qasm_filename)
+        if not os.path.exists(qasm_file_full):
+            raise FileNotFoundError(f"QASM file not found: {qasm_file_full}")
+
+        with open(qasm_file_full, 'r', encoding='utf-8') as f:
+            qasm_content = f.read()
+
+        if debugging:
+            logger.info('[DYNEX-CIRCUIT-BACKEND] Converting qasm->dynexCircuit')
+
+        # Convert QASM to DynexCircuit
+        dynex_circuit = _qasm_to_circuit(qasm_content, debugging)
+
+        # Prepare serialization of model
+        feed_dict = {str(k): v for k, v in dynex_circuit.placeholders.items()}
+        model = dynex_circuit.hamiltonian.compile()
+
+        # Generate .model file (same format as dynex_circuit_backend.py)
+        _q = model.to_qubo(index_label=False, feed_dict=feed_dict)
+        model_dict = {
+            'q': _compress(str(_q[0])),
+            'offset': _q[1]
+        }
+
+        # Save .model file (same format as dynex_circuit_backend.py lines 175-181)
+        model_file = qasm_file_full + '.model'
+        with open(model_file, 'w', encoding="utf-8") as f:
+            # Match exact format from dynex_circuit_backend.py
+            f.write('{"feed_dict":"')
+            f.write(str(feed_dict))
+            f.write('",')
+            f.write('"model":')
+            f.write('{"q":"')
+            f.write(_compress(str(_q[0])))
+            f.write('","offset":')
+            f.write(str(_q[1]))
+            f.write('}')
+            f.write('}')
+
+        if debugging:
+            logger.info(f'[DYNEX-CIRCUIT-BACKEND] Model written to {model_file}')
+
+        return feed_dict, model_dict
+
     @staticmethod
     def check_pennylane_circuit(circuit) -> bool:
         if isinstance(circuit, qml.QNode):
@@ -344,7 +482,7 @@ class DynexCircuit:
             # Compute circuit on Dynex:
             import dynex_circuit
             measure = dynex_circuit.execute(circuit, params, mainnet=True)
-            print(measure)
+            self.logger.info(str(measure))
 
             │   DYNEXJOB │   QUBITS │   QUANTUM GATES │   BLOCK FEE │   ELAPSED │   WORKERS READ │   CIRCUITS │   STEPS │   GROUND STATE │
             ├────────────┼──────────┼─────────────────┼─────────────┼───────────┼────────────────┼────────────┼─────────┼────────────────┤
