@@ -26,7 +26,6 @@ STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 import ast
-import models
 import multiprocessing
 import zlib
 import subprocess
@@ -45,9 +44,9 @@ import numpy as np
 from IPython.core.display_functions import clear_output
 from tabulate import tabulate
 
-from config import DynexConfig
-from models import BQM
-from api import DynexAPI
+from dynex.config import DynexConfig
+from dynex.models import BQM
+from dynex.api import DynexAPI
 
 
 def to_wcnf_string(clauses, num_variables, num_clauses):
@@ -308,7 +307,7 @@ class _DynexSampler:
     """
     num_retries: int = 10
 
-    def __init__(self, model: models.ABC, logging=True, mainnet=True, description='Dynex SDK Job', test=False,
+    def __init__(self, model, logging=True, mainnet=True, description='Dynex SDK Job', test=False,
                  bnb=True, filename_override='', config: DynexConfig = None):
 
         if not test and not os.path.isfile('dynex.test'):
@@ -696,13 +695,24 @@ class _DynexSampler:
         """
 
         directory = self.filepath_full
-        fn = self.filename + "."
+        # Get base filename without extension for matching
+        base_name = self.filename.rsplit('.', 1)[0] if '.' in self.filename else self.filename
+        fn_prefix = base_name + "."
 
         # search for current solution files:
         for filename in os.listdir(directory):
-            if filename.startswith(fn):
+            # Match files that start with base name (e.g., "b0a4d50e4224faaca512c7a958f10f2e")
+            # or exactly match self.filename (e.g., "b0a4d50e4224faaca512c7a958f10f2e.dnx")
+            if filename.startswith(fn_prefix) or filename == self.filename:
                 # check if salt already added:
-                if filename.split('.')[-1].isnumeric():
+                # If file ends with .dnx (no salt), add salt
+                # If file already has salt (e.g., filename.dnx.123 or filename.dnx.hexstring), skip
+                parts = filename.split('.')
+                if len(parts) >= 2 and parts[-1] == 'dnx':
+                    # File ends with .dnx, no salt yet - add salt
+                    os.rename(directory + '/' + filename, directory + '/' + filename + '.' + secrets.token_hex(16))
+                elif len(parts) >= 3 and parts[-2] == 'dnx' and parts[-1].isnumeric():
+                    # File has numeric salt (old format), add hex salt
                     os.rename(directory + '/' + filename, directory + '/' + filename + '.' + secrets.token_hex(16))
         return
 
@@ -726,7 +736,8 @@ class _DynexSampler:
 
         # search for current solution files:
         for filename in os.listdir(directory):
-            if filename.startswith(fn) and filename.endswith('model') == False:
+            # Match files that start with filename. (with salt) OR exactly match filename (without salt yet)
+            if (filename.startswith(fn) or filename == self.filename) and filename.endswith('model') == False:
                 if os.path.getsize(directory + '/' + filename) > 0:
                     filtered_files.append(filename)
 
@@ -753,15 +764,24 @@ class _DynexSampler:
         # format: xxx.bin.32.1.0.0.000000
         # jobfile chips steps loc energy
         info = file[len(self.filename) + 1:]
-        chips = int(info.split(".")[0])
-        steps = int(info.split(".")[1])
-        loc = int(info.split(".")[2])
-
-        # energy can also be non decimal:
-        if len(info.split(".")) > 4:
-            energy = float(info.split(".")[3] + "." + info.split(".")[4])
+        # Check if info contains metrics (numeric values) or just salt (hex string)
+        parts = info.split(".")
+        # If first part is not numeric, it's likely just salt - use defaults
+        if len(parts) >= 4 and parts[0].isnumeric():
+            chips = int(parts[0])
+            steps = int(parts[1])
+            loc = int(parts[2])
+            # energy can also be non decimal:
+            if len(parts) > 4 and parts[4].isnumeric():
+                energy = float(parts[3] + "." + parts[4])
+            else:
+                energy = float(parts[3])
         else:
-            energy = float(info.split(".")[3])
+            # File has salt but no metrics yet - use defaults (solver may still be processing)
+            chips = 0
+            steps = 0
+            loc = 0
+            energy = 0.0
 
         with open(self.filepath + file, 'r') as ffile:
             data = ffile.read()
@@ -805,12 +825,12 @@ class _DynexSampler:
 
     # list and download solution files ================================================================================================
     def download_and_validate(self, ftp, name, local_path, max_attempts=3):
-        """Вспомогательная функция для загрузки и валидации файла"""
+        """Helper function for downloading and validating file"""
         for attempt in range(max_attempts):
             try:
                 with open(local_path, 'wb') as file:
                     if self.logging and attempt > 0:
-                        self.logger.info(f'[DYNEX] Попытка {attempt + 1}: Загрузка файла {name}')
+                        self.logger.info(f'[DYNEX] Attempt {attempt + 1}: Downloading file {name}')
                     ftp.retrbinary(f'RETR {name}', file.write)
 
                 if os.path.getsize(local_path) > 0 and self.validate_file(name, self.logging):
@@ -818,16 +838,16 @@ class _DynexSampler:
                     return True
 
                 if self.logging:
-                    self.logger.info(f'[DYNEX] Удаление файла {name} (некорректная энергия или напряжение)')
+                    self.logger.info(f'[DYNEX] Removing file {name} (incorrect energy or voltage)')
                 os.remove(local_path)
                 self.api.report_invalid(filename=name, reason='wrong energy reported')
 
             except Exception as e:
                 if self.logging:
-                    self.logger.error(f'[DYNEX] Ошибка при загрузке {name}: {str(e)}')
+                    self.logger.error(f'[DYNEX] Error downloading {name}: {str(e)}')
                 if attempt == max_attempts - 1:
                     raise
-                time.sleep(1)  # Короткая пауза перед повторной попыткой
+                time.sleep(1)  # Short pause before retry
         return False
 
     def list_files_with_text(self):
@@ -848,9 +868,9 @@ class _DynexSampler:
 
             with FTP(host) as ftp:
                 ftp.login(user=self.config.ftp_username, passwd=self.config.ftp_password)
-                ftp.cwd("")  # Корневая директория
+                ftp.cwd("")  # Root directory
 
-                # Используем списковое включение для фильтрации файлов
+                # Use list comprehension to filter files
                 valid_files = [
                     (name, facts) for name, facts in ftp.mlsd()
                     if 'size' in facts
@@ -861,7 +881,7 @@ class _DynexSampler:
                 for name, _ in valid_files:
                     local_path = os.path.join(self.filepath, name)
 
-                    # Пропускаем существующие валидные файлы
+                    # Skip existing valid files
                     if os.path.isfile(local_path) and os.path.getsize(local_path) > 0:
                         continue
 
@@ -870,7 +890,7 @@ class _DynexSampler:
 
         except Exception as e:
             if self.logging:
-                self.logger.error(f'[DYNEX] Ошибка FTP: {str(e)}')
+                    self.logger.error(f'[DYNEX] FTP error: {str(e)}')
             raise
 
     # clean function ======================================================================================================================
@@ -1057,7 +1077,7 @@ class _DynexSampler:
                     self.clauses = self.model.clauses
                 self._save_cnf(self.clauses, self.filepath + self.filename)
             if self.type == 'wcnf':
-                self.save_wcnf(self.clauses, self.filepath + self.filename, self.num_variables, self.num_clauses,
+                self._save_wcnf(self.clauses, self.filepath + self.filename, self.num_variables, self.num_clauses,
                                self.model.var_mappings)
 
                 # aggregate sampleset:
@@ -1066,6 +1086,60 @@ class _DynexSampler:
 
         return retval
 
+    def read_voltage_data(self, file, mainnet, rank):
+        file_path = os.path.join(self.filepath, file)
+        try:
+            with open(file_path, 'r') as ffile:
+                # Determine reading strategy
+                if mainnet:
+                    if rank > 1:
+                        return self._read_last_non_empty_line(ffile)
+                    else:
+                        return self._read_second_line(ffile)
+                else:  # test-net
+                    if rank > 1:
+                        return self._read_last_non_empty_line(ffile)
+                    else:
+                        return self._read_entire_file(ffile)
+
+        except (IOError, OSError) as e:
+            self.logger.error(f'Error reading file {file_path}: {e}')
+            return ['NaN']
+
+    def _read_last_non_empty_line(self, file_obj):
+        last_line = None
+        for line in file_obj:
+            if line.strip():
+                last_line = line
+
+        if not last_line:
+            return ['NaN']
+
+        return self._process_voltage_line(last_line)
+
+    def _read_second_line(self, file_obj):
+        file_obj.readline()
+        second_line = file_obj.readline()
+
+        if not second_line:
+            return ['NaN']
+
+        return self._process_voltage_line(second_line)
+
+    def _read_entire_file(self, file_obj):
+        data = file_obj.read()
+        return self._process_voltage_line(data)
+
+    @staticmethod
+    def _process_voltage_line(line):
+        if not line:
+            return ['NaN']
+
+        voltages = line.split(", ")[:-1]
+        if voltages and not voltages[-1]:
+            voltages = voltages[:-1]
+        return voltages if voltages else ['NaN']
+
     # main sampling function =================================================================================================================
     def _sample(self, num_reads=32, annealing_time=10, switchfraction=0.0, alpha=20, beta=20, gamma=1, delta=1,
                 epsilon=1, zeta=1, minimum_stepsize=0.05, debugging=False, block_fee=0, is_cluster=False, shots=1,
@@ -1195,27 +1269,37 @@ class _DynexSampler:
                     localtype = 0
                 elif self.type == 'qasm':
                     localtype = 5
-                    # testnet qasm sampling requires a dedicated library (not in default package):
-                    command = 'python3 dynex_circuit_backend.py --mainnet False --file ' + self.model.qasm_filepath + self.model.qasm_filename
-                    if debugging:
-                        command = command + ' --debugging True'
-                    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
-                    if debugging:
-                        for c in iter(lambda: process.stdout.read(1), b""):
-                            sys.stdout.write(c.decode('utf-8'))
-                    elif self.logging:
-                        self.logger.info("[DYNEX] *** WAITING FOR READS ***")
-                        process.wait()
-                    # read returned model:
-                    f = open(self.model.qasm_filepath + self.model.qasm_filename + '.model', "r", encoding="utf-8")
-                    _data = json.load(f)
-                    _feed_dict = _data['feed_dict']
-                    _model = _data['model']
+                    # testnet qasm sampling: convert QASM to BQM using DynexCircuit
+                    # This replaces the need for external dynex_circuit_backend.py script
+                    if self.logging:
+                        self.logger.info("[DYNEX] Converting QASM circuit to BQM for testnet...")
+                    
+                    try:
+                        from dynex.dynex_circuit import DynexCircuit
+                        # Use DynexCircuit method to convert QASM to BQM
+                        _feed_dict, _model = DynexCircuit._convert_qasm_to_bqm(
+                            self.model.qasm_filepath,
+                            self.model.qasm_filename,
+                            debugging=debugging
+                        )
+                    except Exception as e:
+                        error_msg = f"Failed to convert QASM circuit to BQM: {str(e)}"
+                        if self.logging:
+                            self.logger.error(f"[DYNEX] ERROR: {error_msg}")
+                        raise Exception(error_msg) from e
+                    
                     if debugging:
                         self.logger.debug('[DYNEX] feed_dict:')
                         self.logger.debug(_feed_dict)
                         self.logger.debug('[DYNEX] model:')
                         self.logger.debug(_model)
+                    
+                    # Read .model file that was created by convert_qasm_to_bqm
+                    # (for compatibility with existing code)
+                    f = open(self.model.qasm_filepath + self.model.qasm_filename + '.model', "r", encoding="utf-8")
+                    _data = json.load(f)
+                    _feed_dict = _data['feed_dict']
+                    _model = _data['model']
                     f.close()
                     # construct circuit model:
                     q = zlib.decompress(bytearray.fromhex(_model['q']))
@@ -1234,7 +1318,7 @@ class _DynexSampler:
                         self.clauses = _model.bqm.to_qubo()
                     self.var_mappings = _model.var_mappings
                     self.precision = _model.precision
-                    self.save_wcnf(self.clauses, self.filepath + self.filename, self.num_variables, self.num_clauses,
+                    self._save_wcnf(self.clauses, self.filepath + self.filename, self.num_variables, self.num_clauses,
                                    self.var_mappings)
 
                 job_id = -1
@@ -1278,7 +1362,7 @@ class _DynexSampler:
                     # command += " mutation_rate=10"
                     command += " init_dt=" + str(minimum_stepsize)
                     command += " cpu_threads=4"
-                    # command += " shots=" + str(rank)
+                    command += " shots=" + str(rank)
                     # self.logger.info(f'[DYNEX DEBUG] Solver command: {command}')
 
                 for shot in range(0, shots):
@@ -1335,19 +1419,29 @@ class _DynexSampler:
                         energy = 0
                     elif self.type in ['wcnf', 'qasm']:
                         info = file[len(self.filename) + 1:]
-                        chips = int(info.split(".")[0])
-                        steps = int(info.split(".")[1])
-                        loc = int(info.split(".")[2])
-                        # energy can also be non decimal:
-                        if len(info.split(".")) > 4 and info.split(".")[4].isnumeric():
-                            energy = float(info.split(".")[3] + "." + info.split(".")[4])
+                        # Check if info contains metrics (numeric values) or just salt (hex string)
+                        parts = info.split(".")
+                        # If first part is not numeric, it's likely just salt - use defaults
+                        if len(parts) >= 4 and parts[0].isnumeric():
+                            chips = int(parts[0])
+                            steps = int(parts[1])
+                            loc = int(parts[2])
+                            # energy can also be non decimal:
+                            if len(parts) > 4 and parts[4].isnumeric():
+                                energy = float(parts[3] + "." + parts[4])
+                            else:
+                                energy = float(parts[3])
                         else:
-                            energy = float(info.split(".")[3])
+                            # File has salt but no metrics yet - use defaults (solver may still be processing)
+                            chips = 0
+                            steps = 0
+                            loc = 0
+                            energy = 0.0
 
                     if mainnet:
                         self.cnt_solutions = cnt_workers
                     else:
-                        self.cnt_solutions = + 1
+                        self.cnt_solutions += 1
                     total_chips += chips
                     total_steps = steps
                     if energy < lowest_energy:
@@ -1386,6 +1480,14 @@ class _DynexSampler:
                     if runupdated == False and mainnet:
                         self.api.update_job_api(job_id)
                         runupdated = True
+                
+                # Sleep in local mode to avoid busy waiting (same as old version)
+                if not mainnet:
+                    time.sleep(1)
+                
+                # Sleep in local mode to avoid busy waiting
+                if not mainnet:
+                    time.sleep(1)
 
             # update mallob - job finished: -------------------------------------------------------------------------------------------------
             if mainnet:
@@ -1439,31 +1541,31 @@ class _DynexSampler:
                 # jobfile chips steps loc energy
                 elif self.type in ['wcnf', 'qasm']:
                     info = file[len(self.filename) + 1:]
-                    chips = int(info.split(".")[0])
-                    steps = int(info.split(".")[1])
-                    loc = int(info.split(".")[2])
-
-                    # energy can also be non decimal:
-                    if len(info.split(".")) > 4 and info.split(".")[4].isnumeric():
-                        energy = float(info.split(".")[3] + "." + info.split(".")[4])
-                    else:
-                        energy = float(info.split(".")[3])
+                    # Try to parse metrics - if file has no metrics yet (only salt), use defaults
+                    parts = info.split(".")
+                    try:
+                        if len(parts) >= 4 and parts[0].isnumeric():
+                            chips = int(parts[0])
+                            steps = int(parts[1])
+                            loc = int(parts[2])
+                            # energy can also be non decimal:
+                            if len(parts) > 4 and parts[4].isnumeric():
+                                energy = float(parts[3] + "." + parts[4])
+                            else:
+                                energy = float(parts[3])
+                        else:
+                            raise ValueError("No metrics in filename")
+                    except (ValueError, IndexError):
+                        # File has salt but no metrics yet - use defaults (solver may still be processing)
+                        chips = 0
+                        steps = 0
+                        loc = 0
+                        energy = 0.0
 
                 total_chips = total_chips + chips
                 total_steps = steps
 
-                with open(self.filepath + file, 'r') as ffile:
-                    data = ffile.read()
-                    # enough data?
-                    if mainnet:
-                        if len(data) > 96:
-                            wallet = data.split("\n")[0]
-                            tmp = data.split("\n")[1]
-                            voltages = tmp.split(", ")[:-1]
-                        else:
-                            voltages = ['NaN']  # invalid file received
-                    else:  # test-net is not returning wallet
-                        voltages = data.split(", ")[:-1]
+                voltages = self.read_voltage_data(file, mainnet, rank)
 
                 # valid result? ignore Nan values and other incorrect data
                 if self.type == 'cnf':
@@ -1478,7 +1580,7 @@ class _DynexSampler:
 
                 if self.type in ['wcnf', 'qasm']:
                     if self.config.solver_version == 1:
-                        if 0 < len(voltages) == self.num_variables and voltages[0] != 'NaN':
+                        if len(voltages) > 0 and voltages[0] != 'NaN' and self.num_variables == len(voltages):
                             sampleset.append(
                                 ['sample', voltages, 'chips', chips, 'steps', steps, 'falsified softs', loc, 'energy',
                                  energy])
@@ -1487,11 +1589,10 @@ class _DynexSampler:
                             if energy < lowest_energy:
                                 lowest_energy = energy
                                 lowest_set = voltages
-                            # add voltages to dimod return sampleset:
+                            # Add voltages to dimod return sampleset (v1: exclude validation vars)
                             dimodsample = {}
-                            i = 0
-                            for var in range(0, self.num_variables - 8):  # REMOVE VALIDATION VARS
-                                # mapped variable?
+                            for i, var in enumerate(range(0, self.num_variables - 8)):
+                                # Mapped variable?
                                 if var in self.var_mappings:
                                     dimodsample[self.var_mappings[var]] = 1
                                     if float(voltages[i]) < 0:
@@ -1500,13 +1601,11 @@ class _DynexSampler:
                                     dimodsample[i] = 1
                                     if float(voltages[i]) < 0:
                                         dimodsample[i] = 0
-                                i = i + 1
-
                             dimod_sample.append(dimodsample)
-
                         else:
                             self.logger.info(f'[DYNEX] OMITTED SOLUTION FILE: {file} - INCORRECT DATA')
                     else:
+                        # v2: process all variables
                         sampleset.append(
                             ['sample', voltages, 'chips', chips, 'steps', steps, 'falsified softs', loc, 'energy',
                              energy])
@@ -1515,21 +1614,18 @@ class _DynexSampler:
                         if energy < lowest_energy:
                             lowest_energy = energy
                             lowest_set = voltages
-                        # add voltages to dimod return sampleset:
+                        # Add voltages to dimod return sampleset (v2: all variables)
                         dimodsample = {}
-                        i = 0
-                        for var in range(0, self.num_variables):
-                            # mapped variable?
+                        for i, var in enumerate(range(0, self.num_variables)):
+                            # Mapped variable?
                             if var in self.var_mappings:
                                 dimodsample[self.var_mappings[var]] = 1
-                                if float(voltages[i]) < 0:
+                                if i < len(voltages) and float(voltages[i]) < 0:
                                     dimodsample[self.var_mappings[var]] = 0
                             else:
                                 dimodsample[i] = 1
-                                if float(voltages[i]) < 0:
+                                if i < len(voltages) and float(voltages[i]) < 0:
                                     dimodsample[i] = 0
-                            i = i + 1
-
                         dimod_sample.append(dimodsample)
 
             if self.type in ['wcnf', 'qasm']:
@@ -1575,6 +1671,7 @@ class _DynexSampler:
             if mainnet:
                 self.api.cancel_job_api(job_id)
             self.logger.info(f"[DYNEX] Exception encountered during hadling exception: {e}")
+            raise e
             return {'error': 'Exception encountered during handling excepiton', 'details': e}
 
         self.sampleset = sampleset_clean
