@@ -33,7 +33,7 @@ import time
 import dimod
 
 from dynex import DynexConfig, BQM, DynexSampler
-from sampler import _DynexSampler
+from .sampler import _DynexSampler
 
 
 def calculate_sha3_256_hash(string):
@@ -151,7 +151,7 @@ def sample_qubo(Q, offset=0.0, logging=True, formula=2, mainnet=False, descripti
 
     """
     dyn_ver = 2 if v2 else 1  # FIXME Temporary solution
-    config = dynex.DynexConfig(mainnet=mainnet, solver_version=dyn_ver)
+    config = DynexConfig(mainnet=mainnet, solver_version=dyn_ver)
     bqm = dimod.BinaryQuadraticModel.from_qubo(Q, offset)
     model = BQM(bqm, logging=logging, formula=formula, config=config)
     sampler = DynexSampler(model, logging=logging, description=description, bnb=bnb, config=config)
@@ -165,7 +165,7 @@ def sample_qubo(Q, offset=0.0, logging=True, formula=2, mainnet=False, descripti
 def sample_ising(h, j, logging=True, formula=2, description='Dynex SDK Job', bnb=True,
                  num_reads=32, annealing_time=10, clones=1, switchfraction=0.0, alpha=20, beta=20, gamma=1, delta=1,
                  epsilon=1, zeta=1, minimum_stepsize=0.05, debugging=False, block_fee=0, is_cluster=True,
-                 shots=1, cluster_type=1, config: DynexConfig=DynexConfig()):
+                 shots=1, cluster_type=1, config: DynexConfig=None):
     """
     Samples an Ising problem.
 
@@ -216,6 +216,8 @@ def sample_ising(h, j, logging=True, formula=2, description='Dynex SDK Job', bnb
     - Returns a dimod sampleset object class:`dimod.sampleset`
 
     """
+    if config is None:
+        config = DynexConfig(mainnet=True)
     bqm = dimod.BinaryQuadraticModel.from_ising(h, j)
     model = BQM(bqm, logging=logging, formula=formula, config=config)
     sampler = DynexSampler(model, logging=logging, description=description, bnb=bnb, config=config)
@@ -225,12 +227,12 @@ def sample_ising(h, j, logging=True, formula=2, description='Dynex SDK Job', bnb
                                block_fee=block_fee, is_cluster=is_cluster, shots=shots, cluster_type=cluster_type)
     return sampleset
 
-def test():
+def test(mainnet=False):
     """
     Performs test of the dynex.ini settings. Successful completion is required to start using the sampler.
     """
     allpassed = True
-    config = DynexConfig()
+    config = DynexConfig(mainnet=mainnet)
     config.logger.info('[DYNEX] TEST: dimod BQM construction...')
 
     bqm = dimod.BinaryQuadraticModel({'a': 1.5}, {('a', 'b'): -1}, 0.0, 'BINARY')
